@@ -58,6 +58,12 @@ class Compiler:
             # watbia-ctl {args}
             "^watiba-ctl \s*(\S.*)": self.watiba_ctl_args,
 
+            # hook-cmd "command pattern" {function: {parms}, function: {parms}}
+            ".*?post-hook-cmd \s*(\S.*) (\S.*) (\S.*)": self.post_hook_generator,
+
+            # hook-cmd-nr (non-recursive) "command pattern" {function: {parms}, function: {parms}}
+            ".*?post-hook-cmd-nr \s*(\S.*) (\S.*) (\S.*)": self.post_hook_nr_generator,
+
             # hook-cmd "command pattern" {function: {parms}, function: {parms}}
             ".*?hook-cmd \s*(\S.*) (\S.*) (\S.*)": self.hook_generator,
 
@@ -108,7 +114,7 @@ class Compiler:
             f'{parms["match"].group(1).strip()},' 
             f'{parms["match"].group(2).strip()},' 
             f'{parms["match"].group(3).strip()},' 
-            'True)')
+            'recursive=True, post=False)')
     
     # Generate command non-recursive hook
     def hook_nr_generator(self, parms):
@@ -116,8 +122,26 @@ class Compiler:
             f'{parms["match"].group(1).strip()},' 
             f'{parms["match"].group(2).strip()},' 
             f'{parms["match"].group(3).strip()},' 
-            'False)')
+            'recursive=False, post=False)')
+
+    # Generate post-command hook
+    def post_hook_generator(self, parms):
+        self.output.append(f'{parms["indentation"]}{watiba_ref}.add_hook('
+            f'{parms["match"].group(1).strip()},' 
+            f'{parms["match"].group(2).strip()},' 
+            f'{parms["match"].group(3).strip()},' 
+            'recursive=True, post=True)')
     
+    # Generate post_command non-recursive hook
+    def post_hook_nr_generator(self, parms):
+        self.output.append(f'{parms["indentation"]}{watiba_ref}.add_hook('
+            f'{parms["match"].group(1).strip()},' 
+            f'{parms["match"].group(2).strip()},' 
+            f'{parms["match"].group(3).strip()},' 
+            'recursive=False, post=True)')
+    
+
+
     # Generate removal of command hooks
     def remove_hooks_generator(self, parms):
         if len(parms["match"].groups()) >  0 and parms["match"].group(1) != None:
