@@ -36,7 +36,7 @@ class Watiba(Exception):
         self.parms = {"ssh-port":22}
         self.hooks = {}
         self.hook_flags = {}
-        self.hook_mode = False
+        self.active_patterns = {}
 
     def set_parms(self, args):
         for k,v in args.items():
@@ -48,7 +48,7 @@ class Watiba(Exception):
     def execute(self, command, host="localhost"):
         context = False
         if host == "localhost":
-            return self.bash(command, context)
+            return self.bash(command, context, run_post_hooks=False)
         else:
             # A simple wrapper for self.bash()
             return self.ssh(command, host)
@@ -59,10 +59,11 @@ class Watiba(Exception):
         return self.bash(f'ssh -p {self.parms["ssh-port"]} {host} "{command}"', context)
 
     # command - command string to execute
-    # context = track or not track current dir
+    # context - track or not track current dir
+    # run_post_hooks - allows spawned threads to avoid running post-hooks
     # Returns:
     #   WTOutput object that encapsulates stdout, stderr, exit code, etc.
-    def bash(self, command, context=True):
+    def bash(self, command, context=True, run_post_hooks=True):
 
         # In order to be thread-safe in the generated code, ALWAYS create a new output object for each command
         #  This is because in the generated code, the object reference, "_watiba_", is global and needs to be in scope
@@ -70,8 +71,12 @@ class Watiba(Exception):
         # singleton with no side effects.
         out = WTOutput()
 
+
+        ##############################################################################################################
+        #                                           PRE-HOOKS
+        ##############################################################################################################
         # Run any command hooks defined for this command
-        results = self.run_hooks(command)
+        results = self.run_hooks(command, post_hook=False)
 
         # Handle any hook failures
         # All hooks are always run, but any one reporting a failure will cause the command to not be run
@@ -80,8 +85,11 @@ class Watiba(Exception):
             out.stderr.append(msg)
             raise Exception(msg)
 
+        ##############################################################################################################
+        #                                           COMMAND
+        ##############################################################################################################
         # Tack on this command to see what the current dir is after the user's command is executed
-        ctx = ' && echo "_watiba_cwd_($(pwd))_"' if context else ''
+        ctx = ' && echo "__watiba_cwd__($(pwd))_"' if context else ''
         p = Popen(f"{command}{ctx}",
                   shell=True,
                   stdout=PIPE,
@@ -96,11 +104,27 @@ class Watiba(Exception):
             # if asked to keep CWD context, find our echo string and remove so the
             # user doesn't see it
             for n, o in enumerate(out.stdout):
-                m = re.match(r'^_watiba_cwd_\((\S.*)\)_$', o)
+                m = re.match(r'^__watiba_cwd__\((\S.*)\)_$', o)
                 if m:
                     os.chdir(m.group(1))
                     del out.stdout[n]
         out.cwd = os.getcwd()
+
+
+        ##############################################################################################################
+        #                                           POST-HOOKS
+        ##############################################################################################################
+        # Run any command post-hooks defined for this command
+        if run_post_hooks:
+            results = self.run_hooks(command, post_hook=True)
+
+        # Handle any post-hook failures
+        # All hooks are always run, but any one reporting a failure will cause the command to not be run
+        if results['success'] != True:
+            msg = f"One or more post-hooks failed. Hooks reporting a problem: {', '.join(results['failed-hooks'])}"
+            out.stderr.append(msg)
+            raise Exception(msg)
+
         return out
 
     def spawn(self, command, resolver, spawn_args, parent_locals, host="localhost"):
@@ -141,12 +165,33 @@ class Watiba(Exception):
 
         return l_promise
     
+
+    # Determine if the command pattern can be run at this time
+    # Return the condition of the regex match() function
+    def is_hook_runnable(self, command_regex, command, post_hook):
+
+        # Avoid a loop on this command pattern
+        # Note: Doesn't matter if the command matches the pattern or not.
+        if self.hook_flags[command_regex]["recursive"] == False and command_regex in self.active_patterns:
+            return None
+        
+        # See if we're to run this hook before or after the command
+        # Note: Doesn't matter if the command matches the pattern or not.
+        if self.hook_flags[command_regex]["post"] != post_hook:
+            return None
+
+        # Tell the caller if the command passed to us matches the regex expression
+        return re.match(command_regex, command)
+
+
     # Run all the command pre-execution hooks
     # If any hooks returns False, meaning it somehow failed (that's determined by the hook)
     # then report so an exception is thrown by the caller
-    def run_hooks(self, command):
+    #
+    # post_hook - False if this is called before the command, True if after
+    def run_hooks(self, command, post_hook=False):
 
-        # object returned to caller:
+        # return_obj - object returned to caller:
         #  "success" - Aggregate True/False of all hooks.  (i.e. any hook that reports False will cause this value to return False)
         #  "failed-hooks" - Array of hook names that reported a False condition
         #       Note: all hooks are called in order even after one reports failure (i.e. reports False)
@@ -155,28 +200,23 @@ class Watiba(Exception):
         # Loop through the hooks and run them.  Also track ones that fail (i.e. report a False return code)
         for command_regex, functions in self.hooks.items():
 
-            # Avoid a loop on this command pattern
-            if self.hook_flags[command_regex]["recursive"] == False and self.hook_mode == True:
-                continue
-
-            # Check if the command passed to us matches the regex expression
-            mat = re.match(command_regex, command)
+            # Are we allowed to run this hook for this command?
+            mat = self.is_hook_runnable(command_regex, command, post_hook)
 
             # Does this command have attached hooks?
             if mat:
-
                 # Yes, command has hooks.  Run them.
                 # If the hook fails track it, but keep going with the other hooks
                 for func, parms in functions.items():
 
-                    # Indicate we're in a hook
-                    self.hook_mode = True
+                    # Track this pattern as an active hook
+                    self.active_patterns[command_regex] = True
 
                     # Call the hook.  The hook must return True if succeeded, False if failed
                     rc = func(mat, parms)
 
-                    # Indicate we're no longer in a hook
-                    self.hook_mode = False
+                    # Remove tracking of this pattern
+                    del self.active_patterns[command_regex]
 
                     # If caller's hook didn't return a bool value, then it is marked as failed
                     rc = False if type(rc) != bool else rc
@@ -239,7 +279,7 @@ class Watiba(Exception):
 
     # Add a new hook.  If command pattern already exists, add the functions to it otherwise create a new pattern level.
     #  Set recursive to True to keep hook from looping because it has a matching command within it
-    def add_hook(self, pattern, function, parms, recursive = True):
+    def add_hook(self, pattern, function, parms, recursive=True, post=False):
 
         defined_pattern = pattern in self.hooks
         defined_function = function in self.hooks[pattern] if defined_pattern else False
@@ -258,16 +298,19 @@ class Watiba(Exception):
         
         # At this point we know the pattern has not been defined yet, so define it
         self.hooks.update({pattern : {function: parms}})
-        self.hook_flags[pattern] = {"recursive": recursive}
+        self.hook_flags[pattern] = {"recursive": recursive, "post": post}
 
     
     # Remove a specific hook, keyed by pattern, or all hooks if no pattern is passed
     def remove_hooks(self, pattern = None):
+
+        # No pattern passed, so nuke it all!
         if not pattern:
             self.hooks = {}
             self.hook_flags = {}
             return
 
+        # Remove the hook for only the pattern passed
         if pattern in self.hooks:
             del self.hooks[pattern]
             del self.hook_flags[pattern]
